require("stategraphs/commonstates")

local actionhandlers =
{    
    ActionHandler(ACTIONS.CHOP, 
        function(inst)
            if not inst.sg:HasStateTag("prechop") then 
                if inst.sg:HasStateTag("chopping") then
                    return "chop"
                else
                    return "chop_start"
                end
            end 
        end),
    ActionHandler(ACTIONS.MINE, 
        function(inst) 
            if not inst.sg:HasStateTag("premine") then 
                if inst.sg:HasStateTag("mining") then
                    return "mine"
                else
                    return "mine_start"
                end
            end 
        end),
}

local events = 
{
    CommonHandlers.OnLocomote(true, false),
    CommonHandlers.OnAttacked(),
    CommonHandlers.OnDeath(),
    CommonHandlers.OnAttack(),
}

local states =
{
    State{
        name = "idle",
        tags = {"idle", "canrotate"},
        onenter = function(inst, pushanim)    
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("idle_shiver_loop", true)
                inst.SoundEmitter:PlaySound("dontstarve/creatures/rook/idle")
        end,

        timeline = 
        {
		    TimeEvent(48*FRAMES, function(inst) inst.SoundEmitter:PlaySound(inst.soundpath .. "sleep") end ),
		    TimeEvent(20*FRAMES, function(inst) inst.SoundEmitter:PlaySound(inst.soundpath .. "idle") end ),
		    TimeEvent(0.2*FRAMES, function(inst) SpawnPrefab("statue_transition").Transform:SetPosition(inst:GetPosition():Get()) end ),
		    TimeEvent(32*FRAMES, function(inst) inst.AnimState:PlayAnimation("idle_inaction_sanity")  end ),
        },
        
        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },

    State{
        name = "run_start",
        tags = {"moving", "running", "canrotate"},
        
        onenter = function(inst)
			inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_pre")
            inst.sg.mem.foosteps = 0
        end,

        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,

        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
        },
        
        timeline=
        {        
            TimeEvent(4*FRAMES, function(inst)
            end),
        },        
        
    },

    State{
        name = "play_onemanband_stomp",
        tags = {"doing", "playing", "idle"},

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("idle_onemanband2_pre")
            inst.AnimState:PushAnimation("idle_onemanband2_loop")
            inst.AnimState:PushAnimation("idle_onemanband2_pst", false)   
        end,

        events=
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end ),
        },
        
    },

    State{
        
        name = "funnyidle",
        tags = {"idle", "canrotate"},
        onenter = function(inst)
        
			
			if inst.components.temperature:GetCurrent() < 5 then
				inst.AnimState:PlayAnimation("idle_shiver_pre")
				inst.AnimState:PushAnimation("idle_shiver_loop")
				inst.AnimState:PushAnimation("idle_shiver_pst", false)
			elseif inst.components.hunger:GetPercent() < TUNING.HUNGRY_THRESH then
                inst.AnimState:PlayAnimation("hungry")
                inst.SoundEmitter:PlaySound("dontstarve/rabbit/beardscream")    
            else
                inst.AnimState:PlayAnimation("idle_inaction")
            end
        end,

        events=
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end ),
        },
        
    },
    

    State{
        name = "run",
        tags = {"moving", "running", "canrotate"},
        
        onenter = function(inst) 
            inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_loop")
            
        end,
        
        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,

        timeline=
        {
            TimeEvent(7*FRAMES, function(inst)
				inst.sg.mem.foosteps = inst.sg.mem.foosteps + 1
                inst.SoundEmitter:PlaySound("dontstarve/creatures/knight/land")
            end),
		    TimeEvent(7*FRAMES, function(inst) SpawnPrefab("statue_transition_2").Transform:SetPosition(inst:GetPosition():Get()) end ),
            TimeEvent(15*FRAMES, function(inst)
				inst.sg.mem.foosteps = inst.sg.mem.foosteps + 1
                inst.SoundEmitter:PlaySound("dontstarve/creatures/knight/land")
            end),
	    TimeEvent(14*FRAMES, function(inst) SpawnPrefab("statue_transition_2").Transform:SetPosition(inst:GetPosition():Get()) end ),
        },

        
        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
        },
    },
    
    
    State{
    
        name = "run_stop",
        tags = {"canrotate", "idle"},
        
        onenter = function(inst)
            inst.AnimState:PlayAnimation("run_pst")
            inst.Physics:Stop()

        end,
        
        onexit = function(inst)
		inst.AnimState:SetBuild("reaper_robotw90")
        end,
        
        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),        
        },
        
    },

    State{
        name = "attack",
        tags = {"attack", "notalking", "abouttoattack", "busy"},
        
        onenter = function(inst)      

            inst.sg.statemem.target = inst.components.combat.target
            inst.components.combat:StartAttack()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("build_loop")
            inst.SoundEmitter:PlaySound("dontstarve/sanity/creature1/attack")
            
            if inst.components.combat.target then
                if inst.components.combat.target and inst.components.combat.target:IsValid() then
                    inst:FacePoint(Point(inst.components.combat.target.Transform:GetWorldPosition()))
                end
            end
            
        end,
        
        timeline=
        {
            TimeEvent(8*FRAMES, function(inst) inst.components.combat:DoAttack(inst.sg.statemem.target) inst.sg:RemoveStateTag("abouttoattack") end),
            TimeEvent(12*FRAMES, function(inst) 
                inst.sg:RemoveStateTag("busy")
            end),               
            TimeEvent(13*FRAMES, function(inst)
                if not inst.sg.statemem.slow then
                    inst.sg:RemoveStateTag("attack")
                end
            end),
            TimeEvent(24*FRAMES, function(inst)
                if inst.sg.statemem.slow then
                    inst.sg:RemoveStateTag("attack")
                end
            end),           
        },
        
        events=
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end ),
        },
    },  

    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.Physics:Stop()
            inst.AnimState:Hide("swap_arm_carry")
            inst.AnimState:PlayAnimation("transform_pre")
		inst.AnimState:SetBuild("reaper_robotw90")
        end,

        events =
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("transform_pst")
                inst:DoTaskInTime(0.01, function() 
                    SpawnPrefab("maxwell_smoke").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("maxwell_smoke").Transform:SetPosition(inst:GetPosition():Get())
                    inst.SoundEmitter:PlaySound("dontstarve/creatures/rook/explo")
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("reapersuit").Transform:SetPosition(inst:GetPosition():Get())
                    inst:Remove()
                end)
            end ),
        },
    },  
   
	State{
		name = "hit",
        tags = {"busy", "hit"},

        onenter = function(inst)
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("jump")
		inst.AnimState:SetBuild("reaper_robotw92")
            inst.components.health:SetInvincible(true)
    		inst:AddTag("shadow")
    		inst:AddTag("shadowcreature")
        end,
        
        onexit = function(inst)
                    SpawnPrefab("statue_transition_2").Transform:SetPosition(inst:GetPosition():Get())
        inst.SoundEmitter:PlaySound("dontstarve/sanity/creature2/dissappear")

        end,
        

        events=
        {
			EventHandler("animover", function(inst)
                if GetWorld().Map then
                
					local max_tries = 8
					for k = 1,max_tries do
						local pos = Vector3(inst.Transform:GetWorldPosition())
						local offset = 13
						pos.x = pos.x + (math.random(3*offset)-offset)          
						pos.z = pos.z + (math.random(3*offset)-offset)
						if GetWorld().Map:GetTileAtPoint(pos:Get()) ~= GROUND.IMPASSABLE then
							inst.Transform:SetPosition(pos:Get() )
							break
						end
					end
                end

				inst.sg:GoToState("appear")
                
			end),
			
        },
    },

    State{
        name = "appear",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.AnimState:PlayAnimation("powerup")
            inst.Physics:Stop()
                    SpawnPrefab("statue_transition").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("statue_transition_2").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
                    SpawnPrefab("crow").Transform:SetPosition(inst:GetPosition():Get())
        inst.SoundEmitter:PlaySound("dontstarve/sanity/creature2/appear")
            inst.components.health:SetInvincible(true)
    		inst:AddTag("shadow")
    		inst:AddTag("shadowcreature")

        end,
        
        onexit = function(inst)
            inst.components.health:SetInvincible(false)
		inst.AnimState:SetBuild("reaper_robotw90")
                    SpawnPrefab("statue_transition_2").Transform:SetPosition(inst:GetPosition():Get())
        inst.SoundEmitter:PlaySound("dontstarve/sanity/creature2/dissappear")
    		inst:RemoveTag("shadow")
    		inst:RemoveTag("shadowcreature")
			inst.components.talker:Say("I RETURN.")
        end,
        
        events = 
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end)
        },
    },

    State{
        name = "stunned",
        tags = {"busy", "canrotate"},

        onenter = function(inst)
            inst:InterruptBufferedAction()
            inst:ClearBufferedAction()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("idle_sanity_pre")
            inst.AnimState:PushAnimation("idle_sanity_loop", true)
            inst.sg:SetTimeout(5)
        end,

        ontimeout = function(inst)
            inst.sg:GoToState("idle")
        end,
    },

        State{ name = "chop_start",
        tags = {"prechop", "chopping", "working"},
        onenter = function(inst)
            inst.equipfn(inst, inst.items["AXE"])
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("chop_pre")

        end,
        
        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("chop") end),
        },
    },
    
    State{
        name = "chop",
        tags = {"prechop", "chopping", "working"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation("chop_loop")        
        end,

        timeline=
        {
            TimeEvent(5*FRAMES, function(inst) 
                    inst:PerformBufferedAction() 
            end),

            TimeEvent(9*FRAMES, function(inst)
                    inst.sg:RemoveStateTag("prechop")
            end),

            TimeEvent(16*FRAMES, function(inst) 
                inst.sg:RemoveStateTag("chopping")
            end),
        },
        
        events=
        {
            EventHandler("animover", function(inst) 
                inst.sg:GoToState("idle")
            end ),            
        },        
    },

    State{ 
        name = "mine_start",
        tags = {"premine", "working"},
        onenter = function(inst)
            inst.equipfn(inst, inst.items["PICK"])
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("pickaxe_pre")
        end,
        
        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("mine") end),
        },
    },
    
    State{
        name = "mine",
        tags = {"premine", "mining", "working"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation("pickaxe_loop")
        end,

        timeline=
        {
            TimeEvent(9*FRAMES, function(inst) 
                inst:PerformBufferedAction() 
                inst.sg:RemoveStateTag("premine") 
                inst.SoundEmitter:PlaySound("dontstarve/wilson/use_pick_rock")
            end),
            
            -- TimeEvent(14*FRAMES, function(inst)
            --     if  inst.sg.statemem.action and 
            --         inst.sg.statemem.action.target and 
            --         inst.sg.statemem.action.target:IsActionValid(inst.sg.statemem.action.action) and 
            --         inst.sg.statemem.action.target.components.workable then
            --             inst:ClearBufferedAction()
            --             inst:PushBufferedAction(inst.sg.statemem.action)
            --     end
            -- end),            
        },
        
        events=
        {
            EventHandler("animover", function(inst) 
                inst.AnimState:PlayAnimation("pickaxe_pst") 
                inst.sg:GoToState("idle", true)
            end ),            
        },        
    },
}

return StateGraph("reaperrobot", states, events, "idle", actionhandlers)